// Copyright (c) 2010 Paul Pogonyshev.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Moved into a separate file because I want to make ARRAY_SIZE() and similar macros work
// fine on data arrays, yet don't want to have multiple source copies.  Having this in
// 'common.hpp' instead gives 'multiple definition' errors.


#ifndef MCT_TESTS_COMMON_DATA_HPP
#define MCT_TESTS_COMMON_DATA_HPP


template <typename type>
struct Data;


template <>
struct Data <int>
{
  static  const int  values1[];
  static  const int  values2[];

  static  int
  generate (int iteration)
  {  return iteration;  }
};

const int  Data <int>::values1[] = { 10, -6, 1, 3 };
const int  Data <int>::values2[] = { 0, -3, 10, 8, 12, 1, -9 };


template <>
struct Data <string>
{
  static  const string  values1[];
  static  const string  values2[];

  static  string
  generate (int iteration)
  {
    string  result;
    result.resize (iteration, '*');
    return result;
  }
};

const string  Data <string>::values1[] = { "mon", "tue", "wed", "thu", "fri", "sat", "sun" };
const string  Data <string>::values2[] = { "jan", "feb", "mar", "apr", "may", "etc" };


template <>
struct Data <foo>
{
  static  const foo  values1[];
  static  const foo  values2[];

  static  foo
  generate (int iteration)
  {
    unsigned int   first = (iteration / 0x100);
    unsigned char  byte3 = (iteration & 0xff);
    unsigned int   byte1 = 0;

    while (first > byte1)
      first -= ++byte1;

    unsigned int  byte2 = first;

    if (byte1 >= 0x100 || byte2 >= 0x100)
      throw logic_error ("iteration is too large");

    return foo (byte1, byte2, byte3);
  }
};

const foo  Data <foo>::values1[]
  = { foo (0, 15, 3), foo (3, 8, 1), foo (9, 2, 2), foo (6, 0, 0), foo (1, 0, 7) };
const foo  Data <foo>::values2[]
  = { foo (1, 2, 1), foo (2, 8, 17), foo (3, 3, 3), foo (19, 0, 2) };


template <>
struct Data <bar>
{
  static  const bar  values1[];
  static  const bar  values2[];

  static  bar
  generate (int iteration)
  {  return bar (iteration);  }
};

const bar  Data <bar>::values1[]
  = { bar (42), bar (0), bar (3), bar (-3) };
const bar  Data <bar>::values2[]
  = { bar (7), bar (13), bar (1), bar (0), bar (88), bar (14) };


#if MCT_HAVE_LONG_LONG

template <>
struct Data <unsigned long long>
{
  static  const unsigned long long  values1[];
  static  const unsigned long long  values2[];

  static  unsigned long long
  generate (int iteration)
  {  return iteration;  }
};

const unsigned long long  Data <unsigned long long>::values1[] = { 10, -6, 1, 3 };
const unsigned long long  Data <unsigned long long>::values2[] = { 0, -3, 10, 8, 12, 1, -9 };

#endif  // MCT_HAVE_LONG_LONG


#endif  // Multi-inclusion guard.


// Local variables:
// mode: c++
// c-basic-offset: 2
// indent-tabs-mode: nil
// fill-column: 90
// End:
