// Copyright (c) 2010 Paul Pogonyshev.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.


// Even though we perform our own integrity validation, and even comparisons to a known
// good implementation in our tests, we still let the tables validate themselves with
// MCT_SELF_VALIDATION.  If nothing else, this ensures that MCT_SELF_VALIDATION behaves
// correctly on valid tables.
#undef  MCT_SELF_VALIDATION
#define MCT_SELF_VALIDATION     1

#include "tests/set-common.hpp"
#include "tests/set-parameters.hpp"


namespace
{
# include "tests/common-data.hpp"
}


BOOST_AUTO_TEST_SUITE (set_iterators)


BOOST_AUTO_TEST_CASE_TEMPLATE (test_iterators_1,
                               parameters, test_parameters)
{
  COMMON_TEST_SETUP;

  set_type  set (ARRAY_RANGE (data::values1));

  {
    size_t  num_elements = 0;
    for (const_iterator iter = set.begin (); iter != set.end (); ++iter)
      ++num_elements;

    BOOST_CHECK_EQUAL (num_elements, set.size ());
  }

  {
    size_t  num_elements = 0;
    for (const_iterator iter = set.end (); iter != set.begin (); --iter)
      ++num_elements;

    BOOST_CHECK_EQUAL (num_elements, set.size ());
  }
}


// Following tests assume MCT_CHECK_PRECONDITIONS mode.

BOOST_AUTO_TEST_CASE_TEMPLATE (test_invalid_iterators_1,
                               parameters, test_parameters_robust_iterator_validation)
{
  COMMON_TEST_SETUP;

  set_type  set (ARRAY_RANGE (data::values1));

  iterator  begin = set.begin ();
  BOOST_CHECK_THROW (--begin, logic_error);

  iterator  end = set.end ();
  BOOST_CHECK_THROW (++end, logic_error);

  type  value;
  BOOST_CHECK_THROW (value = *end, logic_error);
}

BOOST_AUTO_TEST_CASE_TEMPLATE (test_erase_iterator_precondition_1,
                               parameters, test_parameters)
{
  COMMON_TEST_SETUP;

  set_type  set (ARRAY_RANGE (data::values1));
  iterator  it = set.begin ();

  // This validation doesn't require fully robust checking to be available.
  set.erase (it);
  BOOST_CHECK_THROW (set.erase (it), logic_error);
}


BOOST_AUTO_TEST_SUITE_END ()


// Local variables:
// mode: c++
// c-basic-offset: 2
// indent-tabs-mode: nil
// fill-column: 90
// End:
